const mysql = require('mysql')
const { customLogBack } = require('prevapp_error')
const { runParallel } = require('prevapp_parallel_funct')


const MYSQL_SERVER = process.env.host


const makeConnection = params => {
  const host = (params.host) ? params.host : MYSQL_SERVER
  const connectionParams = {
    ...params,
    connectionLimit: 50,
    host,
    multipleStatements: params.multiple,
    queueLimit: 20,
  }
  return mysql.createConnection(connectionParams)
}


const makePool = params => {
  const connectionParams = {
    ...params,
    connectionLimit: 50,
    multipleStatements: params.multiple || false,
  }
  return mysql.createPool(connectionParams)
}

const connect = params => {
  const connectProomise = new Promise(
    (resolve, reject) => {
      const connection = makeConnection(params)
      connection.connect(err => {
        if (err) {
          reject(err)
        }
        resolve(connection)
      })
    },
  )
  return connectProomise
}


const makeAndConnect = async (params, cb) => {
  try {
    const conn = await connect(params)
    cb(null, conn)
  } catch (err) {
    const errLog = customLogBack({
      code: err.code,
      err: 'cant connect to db',
      params,
    }, true)
    cb(errLog)
  }
}

const makeSingleMysqlConnection = async params => {
  const connectProomise = new Promise(
    (resolve, reject) => {
      makeAndConnect(params, (err, connection) => {
        if (err) {
          reject(err)
        } resolve(connection)
      })
    })
  return connectProomise
}

const makeMultipleConnections = (params, cb) => {
  runParallel(params, makeAndConnect, cb)
}

exports.makeAndConnect = makeAndConnect
exports.makeMultipleConnections = makeMultipleConnections
exports.makeSingleMysqlConnection = makeSingleMysqlConnection

const makeMysqlPoolCluster = params => {
  const poolCluster = mysql.createPoolCluster()
  params.map(param => {
    poolCluster.add(param.name, param)
    return null
  },
  )
  return poolCluster
}

const getMysqlClusterConn = async (cluster, name) => {
  const Proomise = new Promise(
    (resolve, reject) => {
      cluster.getConnection(name, (err, connection) => {
        if (err) {
          reject(err)
        } resolve(connection)
      })
    })
  return Proomise
}

exports.makeMysqlPoolCluster = makeMysqlPoolCluster
exports.makePool = makePool
exports.getMysqlClusterConn = getMysqlClusterConn

const execQuery = async (conn, query, params = []) => {
  const Proomise = new Promise(
    (resolve, reject) => {
      conn.query(query, params, (err, results) => {
        if (err) {
          reject(err)
        } resolve(results)
      })
    })
  return Proomise
}

exports.execQuery = execQuery

const getClientDb = async (client, host, cluster) => {
  const {
    dbList,
  } = host
  if (!dbList[client]) {
    return customLogBack({
      err: `Client ${client} does not exist`,
    }, true, false)
  }
  const dbName = dbList[client].databaseName
  const hostName = dbList[client].name
  const conn = await getMysqlClusterConn(cluster, hostName)
  return { conn, dbName }
}

const prepareQuery = async (msg, queryString, client, host, cluster, parsedValues) => {
  const {
    conn,
    dbName,
    err,
  } = await getClientDb(client, host, cluster)
  if (err) { return { err } }
  const queryParts = queryString.split('|')
  const query = ` ${queryParts[0]}${dbName}${queryParts[1]}`
  const result = await execQuery(conn, query, parsedValues)
  // conn.release()
  return result
}


exports.prepareQuery = prepareQuery
